@@ -16,8 +16,10 @@
 # IMPORTS
 # =============================================================================
 
+import copy
 from collections.abc import Mapping
 
+
 # =============================================================================
 # DOC INHERITANCE
 # =============================================================================
@@ -61,10 +63,32 @@ class Bunch(Mapping):
     def __getattr__(self, a):
         """x.__getattr__(y) <==> x.y."""
         try:
-            return self[a]
+            return self._data[a]
         except KeyError:
             raise AttributeError(a)
 
+    def __copy__(self):
+        """x.__copy__() <==> copy.copy(x)."""
+        cls = type(self)
+        return cls(str(self._name), data=self._data)
+
+    def __deepcopy__(self, memo):
+        """x.__deepcopy__() <==> copy.copy(x)."""
+        # extract the class
+        cls = type(self)
+
+        # make the copy but without the data
+        clone = cls(name=str(self._name), data=None)
+
+        # store in the memo that clone is copy of self
+        # https://docs.python.org/3/library/copy.html
+        memo[id(self)] = clone
+
+        # now we copy the data
+        clone._data = copy.deepcopy(self._data, memo)
+
+        return clone
+
     def __iter__(self):
         """x.__iter__() <==> iter(x)."""
         return iter(self._data)
@@ -76,7 +100,7 @@ class Bunch(Mapping):
     def __repr__(self):
         """x.__repr__() <==> repr(x)."""
         content = repr(set(self._data)) if self._data else "{}"
-        return f"{self._name}({content})"
+        return f"<{self._name} {content}>"
 
     def __dir__(self):
         """x.__dir__() <==> dir(x)."""
